;****************************************************************************************
;*                           VOLTMETER VON DL3UKH                                       * 
;*                           ====================                                       *
;*                                                                                      *
;****************************************************************************************

        LIST    P=16F88       
        TITLE   "VOLTMETER"

        ERRORLEVEL -302
        INCLUDE <P16F88.INC>

__CONFIG _CONFIG1, _CP_OFF & _CCPMX_RB0 & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDTE_OFF & _INTRC_IO
__CONFIG _CONFIG2, _IESO_OFF & _FCMEN_OFF


;--------------------------------------------------------------------------
;REGISTER ZUORDNEN NACH DEM DIE FESTEN-REGISTER ZU ENDE SIND
;ES GEHT ALSO ERST AB 22 HEX LOS
;DIES IST ALSO DIE RAM-SPEICHER ZUORDNUNG

HELP0   EQU 0X22       ;HILFSZELLE
HELP1   EQU 0X23       ;HILFSZELLE

;FUER 4-BIT
HELP2   EQU 0X24       ;HILFSZELLE
MERKE   EQU 0X25       ;HILFSZELLE
BIT     EQU 0X26       ;HILFSZELLE


;ANALOG/DIGITAL-WANDLUNG
AD_H    EQU 0X27
AD_L    EQU 0X28

;BCD-ANZEIGE
MERK    EQU  0X29 ;ODER H'...'  FR HILFSZELLE/BYTE                            *
TEM     EQU  0X2A ;ODER H'...'  FR HILFSZELLE/BYTE                            *
ANZ_L   EQU  0X2B ;ODER H'...'  FR HILFSZELLE/BYTE                            *
ANZ_H   EQU  0X2C ;ODER H'...'  FR HILFSZELLE/BYTE                            *
ZAE     EQU  0X2D ;ODER H'...'  FR HILFSZELLE/BYTE                            *
VER     EQU  0X2E ;ODER H'...'  FR HILFSZELLE/BYTE                            *


;------------------------- MACROS -----------------------------------------

BANK0           MACRO                   ;MACRO TO SELECT DATA RAM BANK 0
                BCF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM

BANK1           MACRO                   ;MACRO TO SELECT DATA RAM BANK 1
                BSF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM

BANK2           MACRO                   ;MACRO TO SELECT DATA RAM BANK 2
                BCF     STATUS,RP0
                BSF     STATUS,RP1
                ENDM

BANK3           MACRO                   ;MACRO TO SELECT DATA RAM BANK 3
                BSF     STATUS,RP0
                BSF     STATUS,RP1
                ENDM

;VARIABLEN
BIT7    EQU 7       ; BIT 7
BIT6    EQU 6       ; BIT 6
BIT5    EQU 5       ; BIT 5
BIT4    EQU 4       ; BIT 4
BIT3    EQU 3       ; BIT 3 
BIT2    EQU 2       ; BIT 2
BIT1    EQU 1       ; BIT 1 
BIT0    EQU 0       ; BIT 0


;==================================================
RS      EQU 2       ; PORTA BIT2 UMSCHALTEN BEFEHL/DATEN      BEI WRITE DATEN IST ES HIGH
EN      EQU 4       ; PORTA BIT4 ENABLE FALLENDE FLANKE
;==================================================

;-----------------------------------------------------------------------------
;SPEICHERANFANG BZW. BEGINN

                ORG 0X0000
                GOTO MAIN               ; SPRUNG ZUM HAUPTPROGRAMM
                ORG 0X0004
                RETFIE                  ; BEI EINEM INTERRUPT SOFORT EIN RETURN

                ORG 0X0010
;*****************************************************************************
;*                              HAUPTPROGRAMM                                *
;*                              22.09.11                                     *
;*****************************************************************************
MAIN            CLRF    PORTA                           
                CLRF    PORTB

;----------- INIT ALLE PORT'S ----------------------------------------------------

                BANK1
                MOVLW   B'01111110'             ;0110-8MHZ INTERN-RC / 1-VOM SYSTEM / 1-STABILE FREQUENZ / 1-RC-TAKT-INTERN FUER SYSTEM
                MOVWF   OSCCON

                MOVLW   0X07                    ;COMPARATORS ALLE AUS VOM PORT A
                MOVWF   CMCON

                MOVLW   B'11101011'             ;INIT INPUT/OUTPUT AM PORTA --> ALLES EINGAENGE AUSSER  RA2 RA4 ALS AUSGAENGE 
                MOVWF   TRISA
                MOVLW   B'00001111'             ;INIT I/O PORTB  
                MOVWF   TRISB

;INIT TIMER 0 FUER 1MSEC
                MOVLW   B'10000100'             ;BIT0-2 1=PRESCALER32, BIT3 0=TMR0, BIT4 0=ENTFAELLT, BIT5 0=TAKT VOM SYSTEM, BIT6 0=KEIN RB-INTERRUPT, BIT7 1=PULL-UPS DISABLE 
                MOVWF   OPTION_REG

;AD-INIT
                MOVLW   B'00000010' ; AN1         ;B'00010000'  ;RA4 ALS AD4/AN4
                MOVWF   ANSEL
;AD-INIT
                MOVLW   B'11100000'  ;RECHTSBNDIG B1 / FOSZ=32 B0 / VSS+REF-SPANNUNG B10
                MOVWF   ADCON1
                BANK0
;AD-INIT
                MOVLW   B'11001001'  ;FOSZ=RC B11 / AD1 B001 / GO-IST AUS + ON IST EIN B001
                MOVWF   ADCON0

;ZEITSCHLEIFE VON 100MSEC, MUSS SEIN WEGEN ANSCHALTEN, DANN ERST DIE ERSTEN BEFEHLE
            MOVLW   D'1'
            CALL    ZS100MS


;INITIALISIERUNG DOT-MATRIX-LCD
            MOVLW   H'20'                   ;FUNCTION SET  4-BIT-LAENGE, 1ZEILIG, 5X7FONT
            CALL    BEFEHL

            MOVLW   H'0C'                   ;DISPLAY ON/OFF   DISPLAY ON, CURSOR OFF, BLINK OFF
            CALL    BEFEHL

            MOVLW   H'01'                   ;CLEAR DISPLAY
            CALL    BEFEHL

;ZEITSCHLEIFE WEGEN CLEAR
            MOVLW   D'4'
            CALL    ZS1MS

;----------- AN DOG-MATRIX-AUSGEBEN ------------------------

ANFANG
            MOVLW   H'80'                   ;ADRESSE 0 
            CALL    BEFEHL
            MOVLW   "G"
            CALL    DATEN

            MOVLW   H'81'                   ;ADRESSE 1
            CALL    BEFEHL
            MOVLW   "e"
            CALL    DATEN

            MOVLW   H'82'                   ;ADRESSE 2
            CALL    BEFEHL
            MOVLW   "h"
            CALL    DATEN

            MOVLW   H'83'                   ;ADRESSE 3
            CALL    BEFEHL
            MOVLW   "t"
            CALL    DATEN

            MOVLW   H'84'                   ;ADRESSE 4
            CALL    BEFEHL
            MOVLW   " "
            CALL    DATEN

            MOVLW   H'85'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   "l"
            CALL    DATEN

            MOVLW   H'86'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   "o"
            CALL    DATEN

            MOVLW   H'87'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   "s"
            CALL    DATEN


;3SEC ZEITSCHLEIFE            
            MOVLW   D'30'
            CALL    ZS100MS

            MOVLW   H'01'                   ;CLEAR DISPLAY
            CALL    BEFEHL

;ZEITSCHLEIFE WEGEN CLEAR
            MOVLW   D'4'
            CALL    ZS1MS


AA          CALL    AD

            CALL    HEX_BCD

            MOVLW   D'5'          ;500MSEC ZEITSCHLEIFE / ALLER 100MSEC WIRD GEMESSEN !
            CALL    ZS100MS

            GOTO    AA




;*****************************************************************************
;*                              AD-MESSUNG                                   *
;*****************************************************************************
;AD_ZS
;AD-INIT
;                BANK1
;                MOVLW   B'00000010' ; AN1         ;B'00010000'  ;RA4 ALS AD4/AN4
;                MOVWF   ANSEL
;                CALL    MUE40

;AD-INIT
;                MOVLW   B'10000000'  ;RECHTSBNDIG B1 / FOSZ=32 B0 / VSS+VDD REF-SPANNUNG B00
;                MOVWF   ADCON1
;                BANK0
;                CALL    MUE40

;AD-INIT
;                MOVLW   B'11001001'  ;FOSZ=RC B11 / AD1 B001 / GO-IST AUS + ON IST EIN B001
;                MOVWF   ADCON0
;                CALL    MUE40

;                BCF     PIR1, ADIF 
;                BSF     ADCON0, GO_DONE  ;GEHT LOS DIE MESSUNG
;                CALL    MUE40

;AD1             BTFSS   PIR1, ADIF
;                GOTO    AD1

;                BCF     ADCON0, GO_DONE  ;STOP DER MESSUNG

;AD-ERGEBNIS ABSPEICHERN IN RAM-ZELLEN AD_H UND AD_L
;                MOVF    ADRESH, W
;                MOVWF   AD_H
;                BANK1                               
;                MOVF    ADRESL, W
;                BANK0                               
;                MOVWF   AD_L

;---------
;1024 ZU 256 DEZ. MACHEN
;                BCF     STATUS, C
;                RRF     AD_H, F         ;ZU 512MAX.
;                RRF     AD_L, F

;                BCF     STATUS, C
;                RRF     AD_H, F         ;ZU 256MAX.
;                RRF     AD_L, F

;                MOVLW   0X00
;                SUBWF   AD_L, W
;                BTFSS   STATUS, Z
;                GOTO    A1

;                INCF     AD_L, F      ;UM EINS ERHHEN ALSO 0X01 FR AD_L --->WENN NULL

;A1              MOVF    AD_L, W
;                CALL    ZS1MS

;                MOVLW   D'10'
;                CALL    ZS1MS

;                RETURN
;*****************************************************************************
;*                              AD-MESSUNG                                   *
;*****************************************************************************
AD              BCF     PIR1, ADIF 
                BSF     ADCON0, GO_DONE  ;GEHT LOS DIE MESSUNG
                CALL    MUE40

AD2             BTFSS   PIR1, ADIF
                GOTO    AD2

                BCF     ADCON0, GO_DONE  ;STOP DER MESSUNG

;AD-ERGEBNIS ABSPEICHERN IN RAM-ZELLEN AD_H UND AD_L
                MOVF    ADRESH, W
                MOVWF   AD_H
                BANK1                               
                MOVF    ADRESL, W
                BANK0                               
                MOVWF   AD_L

                RETURN

;*****************************************************************************
;*                              UNTERPROG. AUSGEBEN AUF DOT-LCD-MODUL        *
;*****************************************************************************
LCD         MOVWF   BIT

            MOVF    PORTB, W
            ANDLW   0X0F
            MOVWF   MERKE

            MOVF    BIT, W
            ANDLW   0XF0
            ADDWF   MERKE, W
            MOVWF   PORTB


            BSF     PORTA, EN

;ZEITSCHLEIFE 4SEC
            MOVLW   D'2'                    ;UNGEFAEHR 4SEC ZEITSCHLEIFE
            MOVWF   HELP2
D1          NOP                             ;T=4
            DECFSZ  HELP2, F                ;T=4
            GOTO    D1                      ;T=8    GES.:16TAKTE * 0,0625SEC= 1SEC    *4 DURCHLAEUFE= 4SEC
;ENDE ZEITSCHLEIFE
 
            BCF     PORTA, EN

;---------------------------------

;LOW 4BIT AUSGEBEN
            RLF     BIT, F
            RLF     BIT, F
            RLF     BIT, F
            RLF     BIT, F

            MOVF    PORTB, W
            ANDLW   0X0F
            MOVWF   MERKE

            MOVF    BIT, W
            ANDLW   0XF0
            ADDWF   MERKE, W
            MOVWF   PORTB


            BSF     PORTA, EN

;ZEITSCHLEIFE 4SEC
            MOVLW   D'2'                    
            MOVWF   HELP2
DDD1        NOP                             
            DECFSZ  HELP2, F                
            GOTO    DDD1                    
;ENDE ZEITSCHLEIFE
 
            BCF     PORTA, EN

;ZEITSCHLEIFE 27SEC
            MOVLW   D'14'                   ;UNGEFAEHR 27SEC ZEITSCHLEIFE
            MOVWF   HELP2
DDDD1       NOP
            DECFSZ  HELP2, F                ;WIE OBEN  
            GOTO    DDDD1
;ENDE ZEITSCHLEIFE

            RETURN

;*****************************************************************************
;*                              UNTERPROG. DATEN AUSGEBEN AUF DOT-LCD-MODUL  *
;*****************************************************************************
DATEN       BSF     PORTA, RS               ;ALS DATEN
            CALL    LCD
            RETURN

;*****************************************************************************
;*                              UNTERPROG. BEFEHL AUSGEBEN AUF DOT-LCD-MODUL *
;*****************************************************************************
BEFEHL      BCF     PORTA, RS                ;ALS BEFEHL
            CALL    LCD
            RETURN



;*****************************************************************************
;*                              40S ZEITSCHLEIFE                            *
;*                              31.10.03                                     *
;*****************************************************************************

MUE40       MOVLW   D'40'       ;40SEC SIND JETZT ABER 8*5=40 ==> 40SEC*5=200SEC
            MOVWF   ZAE
MSA1        NOP             ;10X NOP ALSO T=40*0.125S=5S  *8=40S  +T16(4S)
            NOP         
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            DECFSZ  ZAE, F
            GOTO    MSA1
            RETURN

;*****************************************************************************
;*         UNTERPROGRAMM MIT TIMER0 ALS ZEITSCHLEIFE FUER  1MSEC ....        *
;*                                                                           *
;*****************************************************************************
ZS1MS                   MOVWF   HELP0                   ;DER INHALT VON W WIRD IN "HELP1" GELADEN

T01                     MOVLW   D'193'
                        MOVWF   TMR0                    ;DER FESTE RAM-SPEICHER (TMR0) WIRD MIT 240 GELADEN
                        BCF     INTCON, TMR0IF            ;OVERFLOW = 0
                
T02                     BTFSS   INTCON, TMR0IF            ;WENN DIE ZEIT VON 10MSEC ABGELAUFEN IST, WIRD T0IF HIGH (1) UND 
                        GOTO    T02                     ;ES ERFOLGT EIN SPRUNG ZU "DECFSC MERK, F"
                        
                        DECFSZ  HELP0, F                ;ES WIRD DER WERT IN "HELP1" ABWRTS GEZHLT, BIS ER (0) IST UND
                        GOTO    T01                     ;DANN ERFOLGT EIN SPRUNG ZU "RETURN"-ENDE DES UNTERPROGRAMMS
                        RETURN

;ZEITBERECHNUNG FUER 1MS
;PRESCALER 32X  *  SYSTEM 4X  * TAKTZEIT ====> 32*4*0.125SEC=16SEC    /DIE TAKTZEIT ERGIBT SICH AUS DEM INTERNEN OSZILLATOR
;1MS : 16SEC= 62,5 GENOMMEN 63                                              /T = 1 : F  (F)-OSZILLATORFREQUENZ
;ALSO
;16S*D'63'=1,0...MSEC
;TMR0 ALSO D'256'-D'63'=D'193' ENTSPRICHT CA. 1MS
;193 DEZIMAL OBEN EINTRAGEN

;*****************************************************************************
;*         UNTERPROGRAMM MIT TIMER0 ALS ZEITSCHLEIFE FUER  100MSEC           *
;*                                                                           *
;*****************************************************************************
ZS100MS                 MOVWF   HELP0                  

A01                     MOVLW   D'100'
                        MOVWF   HELP1

A03                     MOVLW   D'193'
                        MOVWF   TMR0                    
                        BCF     INTCON, TMR0IF         
                
A02                     BTFSS   INTCON, TMR0IF          
                        GOTO    A02                    
                        
                        DECFSZ  HELP1, F
                        GOTO    A03

                        DECFSZ  HELP0, F               
                        GOTO    A01                     

                        RETURN


;****************************************************************************************
;*                        UNTERPROGRAMM BCD-UMRECHNUNG UND ANZEIGE FR DS18S20          *
;*                        DIES IST NUR MAL GANZ SCHNELL ZUSAMMEN GEPFUSCHT              *
;*                        DESHALB STEHEN NUR GANZ WENIGE KOMMENTARE DRIN                *
;*                        ES FUNKTIONIERT ABER !!!!!!!!                                 *
;*                                                                                      *
;*                        ES WIRD AUF EINE LCD-ANZEIGE 1X8 ANGEZEIGT                    *
;*                                                                                      *
;*                                                                                      *
;*       MAN BENTIGT HIER 7 RAM-SPEICHERZELLE , MAN MUSS DIESE DEFINIEREN AM ANFANG    *
;*       AD_L/TEMP_L  EQU  0X... ODER H'...'  FR TEMPERATUR-ERGEBNIS-LOW-BYTE WERDEN GENUTZT*
;*       AD_H/TEMP_H  EQU  0X... ODER H'...'  FR TEMPERATUR-ERGEBNIS-HIGH-BYTE    "         *
;*       MERK    EQU  0X... ODER H'...'  FR HILFSZELLE/BYTE                            *
;*       TEM     EQU  0X... ODER H'...'  FR HILFSZELLE/BYTE                            *
;*       ANZ_L   EQU  0X... ODER H'...'  FR HILFSZELLE/BYTE                            *
;*       ANZ_H   EQU  0X... ODER H'...'  FR HILFSZELLE/BYTE                            *
;*       ZAE     EQU  0X... ODER H'...'  FR HILFSZELLE/BYTE                            *
;*       VER     EQU  0X... ODER H'...'  FR HILFSZELLE/BYTE                            *
;*                                                                                      *
;*                                                                                      *
;*                              31.03.11                                                *
;****************************************************************************************
HEX_BCD     CLRF    MERK

            MOVF    AD_L, W
            MOVWF   TEM

            CLRF    ANZ_L
            CLRF    ANZ_H

            BTFSS   TEM, BIT0       ;JETZT NICHT VORHER NACH RECHTS-GESCHOBEN
            GOTO    W001
            MOVLW   0X01
            MOVWF   ANZ_L

W001        BTFSS   TEM, BIT1
            GOTO    W002
            MOVF    ANZ_L, W
            ADDLW   0X02
            MOVWF   ANZ_L

W002        BTFSS   TEM, BIT2
            GOTO    W003
            MOVF    ANZ_L, W
            ADDLW   0X04
            MOVWF   ANZ_L

;-------
;BIT3 ENTSPRECHEN DER ZAHL DEZ. 8

W003        BTFSS   TEM, BIT3
            GOTO    W004
            MOVLW   D'8'
            MOVWF   ZAE

W003B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W003A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L

W003A       DECFSZ  ZAE, F      ;8 MAL == INCF ANZ_L == ALSO + 8
            GOTO    W003B

;--------
;BIT4 ENTSPRECHEN DER ZAHL DEZ. 16

W004        BTFSS   TEM, BIT4
            GOTO    W005
            MOVLW   D'16'
            MOVWF   ZAE

W004B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W004A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L

W004A       DECFSZ  ZAE, F      ;16 MAL == INCF ANZ_L == ALSO + 16
            GOTO    W004B

;--------
;BIT5 ENTSPRECHEN DER ZAHL DEZ. 32

W005        BTFSS   TEM, BIT5
            GOTO    W006
            MOVLW   D'32'
            MOVWF   ZAE

W005B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W005A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L 

W005A       DECFSZ  ZAE, F      ;32 MAL == INCF ANZ_L == ALSO + 32
            GOTO    W005B

;--------
;BIT6 ENTSPRECHEN DER ZAHL DEZ. 64

W006        BTFSS   TEM, BIT6
            GOTO    W007
            MOVLW   D'64'
            MOVWF   ZAE

W006B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W006A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L

            MOVF    ANZ_L, W
            ANDLW   0XFF
            MOVWF   VER 
            MOVLW   0XA0
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W006A
            CLRF    ANZ_L
            MOVF    ANZ_H, W
            ANDLW   0X0F        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE // ABER ANZ_H // ANZ_L REICHT NICHT MEHR AUS !!!!
            ADDLW   0X01
            MOVWF   ANZ_H

W006A       DECFSZ  ZAE, F      ;64 MAL == INCF ANZ_L == ALSO + 64
            GOTO    W006B

;--------
;BIT7 ENTSPRECHEN DER ZAHL DEZ. 128

W007        BTFSS   TEM, BIT7
            GOTO    W008
            MOVLW   D'128'
            MOVWF   ZAE

W007B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W007A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L

            MOVF    ANZ_L, W
            ANDLW   0XFF
            MOVWF   VER
            MOVLW   0XA0
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W007A
            CLRF    ANZ_L
            MOVF    ANZ_H, W
            ANDLW   0X0F        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE // ABER ANZ_H // ANZ_L REICHT NICHT MEHR AUS !!!!
            ADDLW   0X01
            MOVWF   ANZ_H

W007A       DECFSZ  ZAE, F      ;128 MAL == INCF ANZ_L == ALSO + 128
            GOTO    W007B



;--------
;BIT0 ENTSPRECHEN DER ZAHL DEZ. 256 VON AD_H

W008        MOVF    AD_H, W
            MOVWF   TEM

            CLRF    MERK

            BTFSS   TEM, BIT0
            GOTO    W009
            MOVLW   D'255'      ; EIGENTLICH 256  +1 FEHLT NOCH
            MOVWF   ZAE

W008B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W008A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L

            MOVF    ANZ_L, W
            ANDLW   0XFF
            MOVWF   VER
            MOVLW   0XA0
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W008A
            CLRF    ANZ_L
            MOVF    ANZ_H, W
            ANDLW   0X0F        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE // ABER ANZ_H // ANZ_L REICHT NICHT MEHR AUS !!!!
            ADDLW   0X01
            MOVWF   ANZ_H

;MAXIMAL BERECHNUNG:
;256+128+64+32+16+8+4+2+1 = 511 ---> ALSO ANZ_H=0X05 UND ANZ_L=0X11  / ZEHNER-STELLE BEI ANZ_H NICHT BERSCHRITTEN


W008A       DECFSZ  ZAE, F      ;255 MAL == INCF ANZ_L == ALSO + 255 / +1 FEHLT NOCH / DA 256
            GOTO    W008B

            BTFSC   MERK, BIT0  ;NUR EINMAL
            GOTO    W009   
;NOCH +1 DA 256DEZ. / SIEHE OBEN DIE NUR 255DEZ.
            MOVLW   D'1'
            MOVWF   ZAE
            BSF     MERK, BIT0  ;MERKER MIT NUR EINMAL
            GOTO    W008B

;--------
;BIT1 ENTSPRECHEN DER ZAHL DEZ. 512 VON AD_H

W009        CLRF    MERK        ;NUR EINMAL ---HIER ZURCK GESETZT

            BTFSS   TEM, BIT1
            GOTO    BCD
            MOVLW   D'255'      ; EIGENTLICH 512  /// ALSO NOCH +255 UND +2 FEHLT NOCH
            MOVWF   ZAE

W009B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W009A
            MOVF    ANZ_L, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE
            ADDLW   0X10
            MOVWF   ANZ_L

            MOVF    ANZ_L, W
            ANDLW   0XFF
            MOVWF   VER
            MOVLW   0XA0
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W009A
            CLRF    ANZ_L
            MOVF    ANZ_H, W
            ANDLW   0X0F        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE // ABER ANZ_H // ANZ_L REICHT NICHT MEHR AUS !!!!
            ADDLW   0X01
            MOVWF   ANZ_H

            MOVF    ANZ_H, W
            ANDLW   0XFF
            MOVWF   VER
            MOVLW   0X0A
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W009A
            CLRF    ANZ_H   ;???? SIEHE OBEN MIT NEIN
            MOVF    ANZ_H, W
            ANDLW   0XF0        ;UEBERTRAG DA 10 ERREICHT SEIN KOENNTE // ABER ANZ_H // ANZ_L REICHT NICHT MEHR AUS !!!!
            ADDLW   0X10
            MOVWF   ANZ_H

;MAXIMAL BERECHNUNG:
;512+256+128+64+32+16+8+4+2+1 = 1023 ---> ALSO ANZ_H=0X10 UND ANZ_L=0X23  / ZEHNER-STELLE BEI ANZ_H BENUTZT


W009A       DECFSZ  ZAE, F      ;255 MAL == INCF ANZ_L == ALSO + 255 / +1 FEHLT NOCH / DA 256
            GOTO    W009B

            BTFSC   MERK, BIT0  ;NUR EINMAL
            GOTO    W01A   
;NOCH +255 DA 512DEZ. / SIEHE OBEN DIE NUR 255DEZ......
            MOVLW   D'255'
            MOVWF   ZAE
            BSF     MERK, BIT0  ;MERKER MIT NUR EINMAL
            GOTO    W009B

W01A        BTFSC   MERK, BIT1  ;NUR EINMAL
            GOTO    BCD
;NOCH +2 DA 512DEZ. / SIEHE OBEN DIE NUR 255DEZ.....
            MOVLW   D'2'
            MOVWF   ZAE
            BSF     MERK, BIT1  ;MERKER MIT NUR EINMAL
            GOTO    W009B


;------------
;------------
;BCD-ANZEIGE
BCD         
            MOVLW   H'80'                   ;ADRESSE 0                  
            CALL    BEFEHL
            MOVLW   0X20                    ;LEERZEICHEN
            CALL    DATEN

;---------------
            CLRF    MERK                  

;---------------
;NEU HINZU
            MOVLW   H'81'                   ;ADRESSE 1                  
            CALL    BEFEHL
            SWAPF   ANZ_H, W
            ANDLW   H'0F'
            ADDLW   H'30'
            MOVWF   VER
            MOVLW   0X30
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W109


            MOVLW   0X20                    ;VORNULL AUSBLENDEN
            CALL    DATEN

            BCF     MERK, BIT0               ;IST AUSGEBLENDETE VORNULL
            GOTO    W110A
            
W109        BSF     MERK, BIT0              ;IST EINE ZAHL
            MOVF    VER, W
            CALL    DATEN

;----------------

W110A       MOVLW   H'82'                   ;ADRESSE
            CALL    BEFEHL
            MOVF    ANZ_H, W
            ANDLW   H'0F'
            ADDLW   H'30'
            MOVWF   VER
            MOVLW   0X30
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W109A

            BTFSC   MERK, BIT0
            GOTO    W109A
            MOVLW   0X20                    ;VORNULL AUSBLENDEN
            CALL    DATEN

            BCF     MERK, BIT0               ;IST AUSGEBLENDETE VORNULL
            GOTO    W110
            
W109A       BSF     MERK, BIT0              ;
            MOVF    VER, W
            CALL    DATEN

;-------------------

W110        MOVLW   H'83'                   ;ADRESSE
            CALL    BEFEHL
            SWAPF   ANZ_L, W
            ANDLW   H'0F'
            ADDLW   H'30'
            MOVWF   VER
            ;MOVLW   0X30
            ;SUBWF   VER, W
            ;BTFSS   STATUS, Z
            ;GOTO    W111

            ;BTFSC   MERK, BIT0              ;WAR VORHER EINE ZAHL BEI ZUM BEISPIEL 0X81 / ALSO NICHT AUSBLENDEN
            ;GOTO    W111                    ;
            ;MOVLW   0X20                    ;VORNULL AUSBLENDEN
            ;CALL    DATEN

            ;BCF     MERK, BIT0               ;IST AUSGEBLENDETE VORNULL
            ;GOTO    W113 
            
;W111        BSF     MERK, BIT0              ;
            MOVF    VER, W
            CALL    DATEN

;----------------

W113        MOVLW   H'85'                   ;ADRESSE 
            CALL    BEFEHL
            MOVF    ANZ_L, W
            ANDLW   H'0F'
            ADDLW   H'30'
            CALL    DATEN

;----------------

            MOVLW   H'86'                   ;ADRESSE               
            CALL    BEFEHL
            MOVLW   0X25                    ;%
            CALL    DATEN

            MOVLW   H'84'                   ;ADRESSE               
            CALL    BEFEHL
            MOVLW   "."                     
            CALL    DATEN

            MOVLW   H'87'                   ;ADRESSE             
            CALL    BEFEHL
            MOVLW   " "                     ;LEERZEICHEN
            CALL    DATEN

            RETURN











            END
